package com.javaonpdas.proxy;

import java.io.*;
import java.text.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import javax.xml.rpc.ParameterMode;
import javax.xml.namespace.QName;
import javax.xml.rpc.encoding.XMLType;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;

import com.javaonpdas.webservices.ImageValue;

public class ImageServiceProxy extends HttpServlet {

  private String[] getNames(URL endPointURL) {
    String[] names = null;
    try {
      // Set up the SOAP Service object
      Service service = new Service();
      Call call = (Call)service.createCall();
      call.setTargetEndpointAddress(endPointURL);
      call.setOperationName(new QName("ImageService",
        "getNames"));
      call.addParameter("extension",
        org.apache.axis.encoding.XMLType.XSD_STRING,
        ParameterMode.IN);
      call.setReturnType(XMLType.SOAP_ARRAY);
      // Invoke the SOAP service
      names = (String[])call.invoke(new Object[] { ".png" });
    }
    catch (AxisFault fault) {
      System.err.println("Generated fault: ");
      System.out.println("  Fault Code   = " +
        fault.getFaultCode());
      System.out.println("  Fault String = " +
        fault.getFaultString());
    }
    catch (Exception e) {
      System.out.println(e.toString());
    }
    return names;
  }

  private ImageValue getImage(URL endPointURL, String name) {
    ImageValue imageValue = null;
    try {
      // Set up the SOAP Service object
      Service service = new Service();
      Call call = (Call)service.createCall();
      call.setTargetEndpointAddress(endPointURL);
      call.setOperationName(new QName("ImageService",
        "getImage"));
      call.addParameter("name",
        org.apache.axis.encoding.XMLType.XSD_STRING,
        ParameterMode.IN);
      QName qn = new QName("urn:BeanService", "ImageValue");
      call.registerTypeMapping(ImageValue.class, qn,
        new BeanSerializerFactory(ImageValue.class, qn),
        new BeanDeserializerFactory(ImageValue.class, qn));
      call.setReturnType(qn);

      // Invoke the SOAP service
      imageValue = (ImageValue)call.invoke(new Object[] { name });
    }
    catch (AxisFault fault) {
      System.err.println("Generated fault: ");
      System.out.println("  Fault Code   = " +
        fault.getFaultCode());
      System.out.println("  Fault String = " +
        fault.getFaultString());
    }
    catch (Exception e) {
      System.out.println(e.toString());
    }
    return imageValue;
  }

  public void doGet(HttpServletRequest request,
    HttpServletResponse response) throws IOException,
    ServletException {

    // Get the value of the service end point parameter
    URL endPointURL = new
      URL(request.getParameter("service-end-point"));
    // Get the value of the action parameter - its value
    // determines any remaining parameters.
    String action = request.getParameter("action");
    // Get the list of image names
    if (action.equalsIgnoreCase("getNames")) {
      String[] names = getNames(endPointURL);
      response.setContentType("text/plain");
      PrintWriter out = response.getWriter();
      if (names != null) {
        for (int i=0; i<names.length; i++)
          out.println(names[i]);
      }
    }
    // Get a specified image
    else if (action.equalsIgnoreCase("getImage")) {
      String name = request.getParameter("name");
      ImageValue imageValue = getImage(endPointURL, name);
      if (imageValue == null) {
        System.out.println("imageValue is null");
      }
      else {
        response.setContentType("text/plain");
        StringBuffer buffer = new StringBuffer();
        buffer.append(""+imageValue.getDate()+"\n");
        buffer.append(imageValue.getEncodedImage()+"\n");
        response.setContentLength(buffer.length());
        PrintWriter out = response.getWriter();
        out.println(buffer.toString());
      }
    }
    else {
      // action not recognised
    }
  }
}
