package com.javaonpdas.webservices.clients.custom;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.ItemStateListener;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.ImageItem;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.io.HttpConnection;
import javax.microedition.io.Connector;

public class ImageViewer extends MIDlet
  implements CommandListener, ItemStateListener {

  private Form mainForm = new Form("");
  private Display display = null;
  private static final String[] LOCATIONS = {"Local", "Remote"};
  private static final String[] NAMES = {};
  private ChoiceGroup nameChoice = null;
  private ChoiceGroup locationChoice =
    new ChoiceGroup("Location",
                    ChoiceGroup.EXCLUSIVE, LOCATIONS, null);
  private Command exitCommand =
    new Command("Exit", Command.EXIT, 10);
  private Command saveCommand =
    new Command("Save", Command.SCREEN, 1);
  private static final String recordStoreName = "images";
  private ImageItem imageItem =
    new ImageItem("Image",
                  Image.createImage(Image.createImage(10,10)),
                  ImageItem.LAYOUT_DEFAULT, "No image yet");
  private byte[] imageByteArray = null;

  public ImageViewer() {
    // populate the list with local names
    String[] names = getNames(true);
    if (names == null)
      nameChoice =
        new ChoiceGroup("Names", ChoiceGroup.EXCLUSIVE,
                        NAMES, null);
    else
      nameChoice =
        new ChoiceGroup("Names", ChoiceGroup.EXCLUSIVE,
                        names, null);
    // add the items to the form
    mainForm.append(nameChoice);
    mainForm.append(locationChoice);
    mainForm.append(imageItem);
    mainForm.addCommand(saveCommand);
    mainForm.addCommand(exitCommand);
    mainForm.setCommandListener(this);
    mainForm.setItemStateListener(this);
  }

  public void startApp() {
    display = Display.getDisplay(this);
    display.setCurrent(mainForm);
  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
  }

  public void commandAction(Command c, Displayable d) {
    Runtime.getRuntime().gc();
    if (c == saveCommand) {
      if (locationChoice.getString(
          locationChoice.getSelectedIndex()).equals("Remote")) {
        System.out.println("saving image");
        // save the current image to the record store
        RecordStore store = null;
        DataOutputStream dos = null;
        DataInputStream dis = null;
        try {
          store =
            RecordStore.openRecordStore(recordStoreName, true);
          System.out.println("size in bytes (before): " +
            store.getSize());
          // create the new record byte array
          ByteArrayOutputStream bos =
            new ByteArrayOutputStream();
          dos = new DataOutputStream(bos);
          String name =
            nameChoice.getString(nameChoice.getSelectedIndex());
          dos.writeUTF(name);
          dos.writeInt(imageByteArray.length);
          dos.write(imageByteArray, 0, imageByteArray.length);
          byte[] ba = bos.toByteArray();
          // check whether the name already exists
          RecordEnumeration re =
            store.enumerateRecords(null, null, false);
          System.out.println("looking for " + name + " in " +
            re.numRecords() + " records");
          boolean found = false;
          int recordID = 1;
          while(re.hasNextElement()) {
            ByteArrayInputStream bis =
              new ByteArrayInputStream(re.nextRecord());
            System.out.println("looking at record "+recordID);
            dis = new DataInputStream(bis);
            if (dis.readUTF().equals(name)) {
              System.out.println(name +
                                 " found match - updating");
              store.setRecord(recordID, ba, 0, ba.length);
              found = true;
              break;
            }
            else
              System.out.println("no match");
              recordID++;
          }
          if (!found) {
            System.out.print(name + " not found - adding " +
              ba.length);
            int addedID = store.addRecord(ba, 0, ba.length);
            System.out.println(" ID = " + addedID);
          }
          System.out.println("size in bytes (after): " +
            store.getSize());
          store.closeRecordStore();
        }
        catch (RecordStoreFullException e) {
          Alert alert = new Alert("Alert");
          alert.setString("The record store is full.");
          alert.setType(AlertType.ERROR);
          alert.setTimeout(Alert.FOREVER);
          display.setCurrent(alert, mainForm);
        }
        catch (Exception e) {
          System.out.println("Error: " + e.toString());
        }
        finally {
          try {
            if (store != null) store.closeRecordStore();
            if (dis != null) dis.close();
            if (dos != null) dos.close(); }
          catch (Exception e) {}
        }
      }
      else {
        // display an alert - can't save from local
        Alert alert = new Alert("Alert");
        alert.setString("It isn't possible to save an image from Local. Save only works from Remote.");
        alert.setType(AlertType.ERROR);
        alert.setTimeout(Alert.FOREVER);
        display.setCurrent(alert, mainForm);
      }
    }
    else if (c == exitCommand) {
      destroyApp(false);
      notifyDestroyed();
    }
  }

  public void itemStateChanged(Item item) {
    Runtime.getRuntime().gc();
    try {
      String location = locationChoice.getString(
        locationChoice.getSelectedIndex());
      System.out.println("location: " + location);
      imageItem.setLabel("Please wait...");
      if (item == locationChoice) {
        String[] names = getNames(location.equals("Local"));
        System.out.println("there are " + names.length +
                           " names");
        int namesInList = nameChoice.size();
        for (int i=0; i<namesInList; i++)
          nameChoice.delete(0);
        for (int i=0; i<names.length; i++)
          nameChoice.append(names[i], null);
        imageItem.setImage(Image.createImage(
          Image.createImage(10,10)));
      }
      else if (item == nameChoice) {
        String name =
          nameChoice.getString(nameChoice.getSelectedIndex());
        imageItem.setImage(Image.createImage(
          Image.createImage(10,10)));
        Image image = getImage(name, location.equals("Local"));
        imageItem.setImage(image);
      }
      imageItem.setLabel("Image");
    }
    catch (Exception e) {
      System.out.println("Error: " + e.toString());
    }
  }

  private String[] getNames(boolean local) {
    String[] names = null;
    RecordStore store = null;
    DataInputStream dis = null;
    HttpConnection connection = null;
    try {
      if (local) {
        store = RecordStore.openRecordStore(recordStoreName,
          true);
        RecordEnumeration re =
          store.enumerateRecords(null, null, false);
        names = new String[re.numRecords()];
        int i=0;
        while(re.hasNextElement()) {
          ByteArrayInputStream bis =
            new ByteArrayInputStream(re.nextRecord());
          dis = new DataInputStream(bis);
          names[i++] = dis.readUTF();
        }
        store.closeRecordStore();
      }
      else {
        String url = "http://192.168.0.3:8080/javaonpdas/servlet/ImageServiceStreamProxy?service-end-point=http://localhost:8080/axis/servlet/AxisServlet&action=getNames";
        connection = (HttpConnection)Connector.open(url);
        connection.setRequestMethod(HttpConnection.GET);
        dis = connection.openDataInputStream();
        int numberOfNames = dis.readInt();
        names = new String[numberOfNames];
        for (int i=0; i<numberOfNames; i++) {
          names[i] = dis.readUTF();
        }
      }
    }
    catch (Exception e) {
      System.out.println("Error (getNames): " + e.toString());
    }
    finally {
      try {
        if (store != null) store.closeRecordStore();
        if (connection != null) connection.close();
        if (dis != null) dis.close();
      } catch (Exception e) {}
    }
    return names;
  }

  private Image getImage(String name, boolean local) {
    Image image = null;
    DataInputStream dis = null;
    if (local) {
      RecordStore store = null;
      try {
        store =
          RecordStore.openRecordStore(recordStoreName, true);
        RecordEnumeration re =
          store.enumerateRecords(null, null, false);
        while(re.hasNextElement()) {
          ByteArrayInputStream bis =
            new ByteArrayInputStream(re.nextRecord());
          dis = new DataInputStream(bis);
          if (dis.readUTF().equals(name)) {
            int imageSize = dis.readInt();
            imageByteArray = new byte[imageSize];
            dis.readFully(imageByteArray);
            image = Image.createImage(imageByteArray,
                                      0, imageByteArray.length);
          }
        }
        store.closeRecordStore();
      }
      catch (Exception e) {
        System.out.println("Error:" + e.toString());
      }
      finally {
        try {
          if (store != null) store.closeRecordStore();
          if (dis != null) dis.close();
        } catch (Exception e) {}
      }
    }
    else {
      HttpConnection connection = null;
      String url = "http://192.168.0.3:8080/javaonpdas/servlet/ImageServiceStreamProxy?service-end-point=http://localhost:8080/axis/servlet/AxisServlet&action=getImage&name="+name;
      try {
        connection = (HttpConnection)Connector.open(url);
        connection.setRequestMethod(HttpConnection.GET);
        int contentLength = (int)connection.getLength();
        if (contentLength>0) {
          dis = connection.openDataInputStream();
          imageByteArray = new byte[contentLength];
          int ch = 0;
          for (int i=0; i<contentLength; i++) {
            if ((ch = dis.read()) != -1) {
              imageByteArray[i] = (byte)ch;
            }
            else {
              System.out.println("Error: encountered EOF");
            }
          }
          image = Image.createImage(imageByteArray, 0,
                                    imageByteArray.length);
        }
      }
      catch (Throwable t) {
        System.out.println("Error:" + t.toString());
        t.printStackTrace();
      }
      finally {
        try {
          if (dis != null) dis.close();
          if (connection != null) connection.close();
        } catch (Exception e) {}
      }
    }
    return image;
  }
}
