package com.javaonpdas.webservices.clients.wingfoot;

import java.util.Date;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

import com.wingfoot.soap.*;
import com.wingfoot.soap.transport.*;
import com.wingfoot.soap.encoding.*;

import com.javaonpdas.SnapShot;

public class SOAPClient extends MIDlet
  implements CommandListener {

  private Form mainForm = new Form("");
  private Display display = null;
  private Command exitCommand =
    new Command("Exit", Command.EXIT, 10);
  private Command getNamesCommand =
    new Command("Get Names", Command.SCREEN, 1);
  private Command getImageCommand =
    new Command("Get Image", Command.SCREEN, 2);
  private TextField textField =
    new TextField(null, null, 10000, TextField.ANY);
  private ImageItem imageItem =
    new ImageItem("Image",
                  Image.createImage(Image.createImage(10,10)),
                  ImageItem.LAYOUT_DEFAULT, "No image yet");

  public SOAPClient() {
    mainForm.append(textField);
    mainForm.append(imageItem);
    mainForm.addCommand(getNamesCommand);
    mainForm.addCommand(getImageCommand);
    mainForm.addCommand(exitCommand);
    mainForm.setCommandListener(this);
  }

  public void startApp() {
    display = Display.getDisplay(this);
    display.setCurrent(mainForm);
  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
  }

  public void commandAction(Command c, Displayable d) {
    if (c == getNamesCommand) {
      try {
        // Prepare the Envelope
        Envelope requestEnvelope = new Envelope();
        requestEnvelope.setBody("extension", ".png");

        // Prepare the call
        Call call = new Call(requestEnvelope);
        call.setMethodName("getNames");
        call.setTargetObjectURI("ImageService");

        // Prepare the transport
        HTTPTransport transport =
          new HTTPTransport("http://192.168.0.3:8080/axis/servlet/AxisServlet",
                            null);
        transport.getResponse(true);

        // Make the call
        Envelope responseEnvelope = call.invoke(transport);

        // Parse the response
        if (responseEnvelope != null) {
          if (responseEnvelope.isFaultGenerated()) {
            Fault f = responseEnvelope.getFault();
            textField.insert("Error: " + f.getFaultString(),
              textField.size());
          }
          else {
            textField.setString(null);
            Object[] parameter =
              (Object[])responseEnvelope.getParameter(0);
            for (int i=0; i<parameter.length; i++)
              textField.insert(parameter[i] + "\n",
                textField.size());
          }
        }
      }
      catch (Exception e) {
        textField.insert("Error:" + e.toString() + "\n",
          textField.size());
      }
    }
    else if (c == getImageCommand) {
      SnapShot snapshot = new SnapShot();
      snapshot.snap();
      try {
        // Prepare the Envelope
        Envelope requestEnvelope = new Envelope();
        requestEnvelope.setBody("name", "kangaroo1.png");

        // Prepare the call
        Call call = new Call(requestEnvelope);
        call.setMethodName("getImage");
        call.setTargetObjectURI("ImageService");
        TypeMappingRegistry registry =
          new TypeMappingRegistry();
        registry.mapTypes("urn:BeanService", "ImageValue",
          new ImageValue().getClass(),
          new BeanSerializer().getClass(),
          new BeanSerializer().getClass());
        call.setMappingRegistry(registry);

        // Prepare the transport
        HTTPTransport transport =
          new HTTPTransport("http://192.168.0.3:8080/axis/servlet/AxisServlet",
                            null);
        transport.getResponse(true);

        // Make the call
        Envelope responseEnvelope = call.invoke(transport);

        // Parse the response
        if (responseEnvelope != null) {
          if (responseEnvelope.isFaultGenerated()) {
            Fault f = responseEnvelope.getFault();
            textField.insert("Error: " + f.getFaultString(),
              textField.size());
          }
          else {
            ImageValue imageValue =
              (ImageValue)responseEnvelope.getParameter(0);
            Date date =
              new Date(imageValue.getDateAsLong().longValue());
            Base64 encodedImage =
              new Base64(imageValue.getEncodedImage());
            imageItem.setImage(Image.createImage(
              encodedImage.getBytes(), 0,
              encodedImage.getBytes().length));
          }
        }
      }
      catch (Exception e) {
        textField.insert("Error:" + e.toString() + "\n",
          textField.size());
        e.printStackTrace();
      }
      snapshot.snap();
    }
    else if (c == exitCommand) {
      destroyApp(false);
      notifyDestroyed();
    }
  }
}
