package com.javaonpdas.webservices.clients.wingfoot;

import java.awt.Frame;
import java.awt.FlowLayout;
import java.awt.Button;
import java.awt.Panel;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.CheckboxGroup;
import java.awt.Checkbox;
import java.awt.List;
import java.awt.MediaTracker;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.util.Date;
import java.io.File;
import java.io.FilenameFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import com.wingfoot.soap.*;
import com.wingfoot.soap.transport.*;
import com.wingfoot.soap.encoding.*;

public class ImageViewer extends Frame
  implements ActionListener, ItemListener {

  private Checkbox localCheckbox = null;
  private Checkbox remoteCheckbox = null;
  private List list = null;
  private ImageCanvas imageCanvas = null;
  private boolean local = true;
  private Button saveButton = null;
  private byte[] imageByteArray = null;
  private Dimension imageCanvasDimension = null;
  private final String LOCAL_DIRECTORY =
    "\\My Documents\\JavaOnPDAs\\resources";
  private final String SOAP_ENDPOINT =
    "http://192.168.0.3:8080/axis/servlet/AxisServlet";

  public ImageViewer(String title) {
    super(title);
    // handle frame closing events
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    } );
    // add components
    setLayout(new GridLayout(2,1));
    Panel topPanel = new Panel();
    CheckboxGroup checkBoxGroup = new CheckboxGroup();
    localCheckbox =
      new Checkbox("Local", checkBoxGroup, true);
    remoteCheckbox =
      new Checkbox("Remote", checkBoxGroup, false);
    localCheckbox.addItemListener(this);
    remoteCheckbox.addItemListener(this);
    topPanel.add(localCheckbox);
    topPanel.add(remoteCheckbox);
    saveButton = new Button("Save");
    saveButton.setEnabled(false);
    topPanel.add(saveButton);
    saveButton.addActionListener(this);
    list = new List(4);
    list.addItemListener(this);
    topPanel.add(list);
    add(topPanel);
    imageCanvas = new ImageCanvas();
    add(imageCanvas);
    String[] names = getNames(true);
    if (names != null)
      for (int i=0; i<names.length; i++)
        list.add(names[i]);
  }

  public void actionPerformed(ActionEvent evt) {
    String cmd = evt.getActionCommand();
    if (cmd.startsWith("Save")) {
      Image image = imageCanvas.getImage();
      String name = list.getSelectedItem();
      FileOutputStream ostream = null;
      try {
        ostream =
          new FileOutputStream(LOCAL_DIRECTORY + "\\" + name);
        ostream.write(imageByteArray);
        ostream.close();
      }
      catch (Exception e) {
        System.out.println("Error:" + e);
      }
      finally {
        try {
          if (ostream != null) ostream.close();
        } catch (Exception e) {}
      }
    }
  }

  public void itemStateChanged(ItemEvent evt) {
    if (evt.getItemSelectable().getClass() ==
      new Checkbox().getClass()) {

      local = localCheckbox.getState();
      if (local) saveButton.setEnabled(false);
      else saveButton.setEnabled(true);
      String[] names = getNames(local);
      list.removeAll();
      if (names != null)
        for (int i=0; i<names.length; i++)
          list.add(names[i]);
    }
    else {
      String name = list.getSelectedItem();
      Image image = getImage(name, local);
      imageCanvas.setImage(image);
      imageCanvas.update(imageCanvas.getGraphics());
    }
  }

  private String[] getNames(boolean local) {
    String[] files = null;
    if (local) {
      FilenameFilter filter = new FilenameFilter() {
        public boolean accept(File dir, String name) {
          return name.endsWith(".jpeg");
        }
      };
      File dir = new File(LOCAL_DIRECTORY);
      files = dir.list(filter);
    }
    else {
      try {
        // Prepare the Envelope
        Envelope requestEnvelope = new Envelope();
        requestEnvelope.setBody("extension", ".jpeg");

        // Prepare the call
        Call call = new Call(requestEnvelope);
        call.setMethodName("getNames");
        call.setTargetObjectURI("ImageService");

        // Prepare the transport
        J2SEHTTPTransport transport =
          new J2SEHTTPTransport(SOAP_ENDPOINT, null);
        transport.getResponse(true);

        // Make the call
        Envelope responseEnvelope = call.invoke(transport);

        // Parse the response
        if (responseEnvelope != null) {
          if (responseEnvelope.isFaultGenerated()) {
            Fault f = responseEnvelope.getFault();
            System.out.println("Error: " + f.getFaultString());
          }
          else {
            Object[] parameter =
              (Object[])responseEnvelope.getParameter(0);
            String[] temp = new String[parameter.length];
            for (int i=0; i<parameter.length; i++)
              temp[i] = (String)parameter[i];
            files = temp;
          }
        }
      }
      catch (java.net.ConnectException e) {
        (new ImageDialog(this, "Error", "Could not connect to " +
                         SOAP_ENDPOINT)).show();
      }
      catch (Exception e) {
        (new ImageDialog(this, "Error", e.toString())).show();
      }
    }
    return files;
  }

  private Image getImage(String name, boolean local) {
    Image image = null;
    if (local) {
      FileInputStream fis = null;
      try {
        // read the file into a byte array
        File imageFile =
          new File(LOCAL_DIRECTORY + "\\" + name);
        if (imageFile.exists()) {
          fis = new FileInputStream(imageFile);
          int length = fis.available();
          imageByteArray = new byte[length];
          fis.read(imageByteArray);
          imageCanvasDimension = imageCanvas.getSize();
          image =
            Toolkit.getDefaultToolkit().createImage(imageByteArray).getScaledInstance(
              -1, imageCanvasDimension.height,
              Image.SCALE_FAST);
          MediaTracker tracker = new MediaTracker(this);
          tracker.addImage(image,0);
          try {tracker.waitForID(0);}
          catch (InterruptedException e){};
        }
      }
      catch (Exception e) {
        (new ImageDialog(this, "Error", e.toString())).show();
      }
      finally {
        try { if (fis != null) fis.close(); }
        catch (Exception e) {}
      }
    }
    else {
      try {
        // Prepare the Envelope
        Envelope requestEnvelope = new Envelope();
        requestEnvelope.setBody("name", name);

        // Prepare the call
        Call call = new Call(requestEnvelope);
        call.setMethodName("getImage");
        call.setTargetObjectURI("ImageService");
        TypeMappingRegistry registry =
          new TypeMappingRegistry();
        registry.mapTypes("urn:BeanService", "ImageValue",
          new ImageValue().getClass(),
          new BeanSerializer().getClass(),
          new BeanSerializer().getClass());
        call.setMappingRegistry(registry);

        // Prepare the transport
        J2SEHTTPTransport transport =
          new J2SEHTTPTransport(SOAP_ENDPOINT, null);
        transport.getResponse(true);

        // Make the call
        Envelope responseEnvelope = call.invoke(transport);

        // Parse the response
        if (responseEnvelope != null) {
          if (responseEnvelope.isFaultGenerated()) {
            Fault f = responseEnvelope.getFault();
            System.out.println("Error: " + f.getFaultString());
          }
          else {
            ImageValue imageValue =
              (ImageValue)responseEnvelope.getParameter(0);
            Date date =
              new Date(imageValue.getDateAsLong().longValue());
            Base64 encodedImage =
              new Base64(imageValue.getEncodedImage());
            imageByteArray = encodedImage.getBytes();
            imageCanvasDimension = imageCanvas.getSize();
            image =
              Toolkit.getDefaultToolkit().createImage(imageByteArray).getScaledInstance(
                -1, imageCanvasDimension.height,
                Image.SCALE_FAST);
            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(image,0);
            try {tracker.waitForID(0);}
            catch (InterruptedException e){};
          }
        }
      }
      catch (Exception e) {
        (new ImageDialog(this, "Error", e.toString())).show();
      }
    }
    return image;
  }

  public static void main(String[] args) {
    Frame f = new ImageViewer("ImageViewer");
    f.setSize(f.getToolkit().getScreenSize());
    f.show();
  }
}
